#ifndef POLYGUN_UTILS_BINARY_UTILS_HPP
#define POLYGUN_UTILS_BINARY_UTILS_HPP

#include <cstdint>

namespace polygun::utils {
	uint16_t bytes_to_uint16(const char* data);
	void uint16_to_bytes(uint16_t data, char* output);
	uint32_t bytes_to_uint32(const char* data);
	void uint32_to_bytes(uint32_t data, char* output);
	float bytes_to_float(const char* data);
	void float_to_bytes(float data, char* output);
}

#endif
